{

// Generic Missile Ambush Monster

GROUP			[Generic] 	// name of monster group
BOXWIDTH		[40] 		// width and depth of bounding box
HOSTILEPLAYER 		[true]		// hostile to player
HOSTILEDIFFERENT 	[false] 	// hostile to different Pawn group
HOSTILESAME 		[false] 	// hostile to same Pawn group
HEALTHATTRIBUTE 	[health] 	// name of health attribute
HEALTH 			[100] 		// initial amount of health
SIGHTDIST 		[500]		// max distance monster can see at idle
ALERTSIGHTDIST 		[600] 		// max distance monster can see when alert
FOV 			[180] 		// field of view in degrees
YAWSPEED 		[120] 		// speed of rotation in deg/sec
DAMAGEATTRIBUTE 	[health]	// attribute damaged by attack
ALERTTRIGGER 		[AlertG] 	// name of alert trigger

STAND 			[Idle] 		// idle animation
TURNL 			[Walk] 		// turn left animation
TURNR 			[Walk] 		// turn right animation

PAIN 			[Injury] 	// pain animations
PAIN1 			[Injury]
PAIN2 			[Injury]
PAIN3 			[Injury]
PAINPERCENT 		[50] 		// percentage of time pain is shown

DIE 			[Die]	 	// dying animations
DIE1 			[Die]
DIE2 			[Die]
DIE3 			[Die]
DIEHEADSHOT		[Die]
DIEHOLD 		[15] 		// time corpse still appears
DIEFADE 		[10] 		// fadeout time of corpse

RUN 			[Run] 		// running animation
RUNSPEED 		[100] 		// average run speed

MISSILEATTACK 		[Punch]		// missile attacking animation
MISSILERANGE 		[300] 		// max distance to start missile attack
PROJECTILE 		[pistol_shell] 	// projectile name
FIREBONE 		[BIP01 R HAND] 	// projectile launch bone
OFFSETX 		[0] 		// launch offsets
OFFSETY 		[0]
OFFSETZ 		[0]
ATTACKDELAY 		[2] 		// time between shots
FIREDELAY 		[0.8] 		// delay after animation starts before projectile launch
SKILL 			[5] 		// skill level 1 to 10
PAWNWEAPON		[pistol]
ATTACKSOUND		[]

LOOTWEAPON		[pistol]	// weapon to give to player when coming near dead body
LOOTAMMO		[pistol_shell]	// ammo to give to player when coming near dead body
LOOTAMMOMIN		[5]		// minimum amount of ammo to give to player
LOOTAMMOMAX		[10]		// maximum amount of ammo to give to player
LOOTSOUND		[]		// sound to play when looted

LOSTTIME 		[15] 		// time to search for enemy before giving up
POINTRADIUS 		[20] 		// radius from point when considered there

HEADBONE		[BIP01 HEAD]	//name of the head bone

// local variables - do not change


RUNFUNC 	[monster_run_start] // monster run to attack function
MISSILEFUNC 	[monster_missile_start] // monster missile function
LOSTFUNC 	[monster_lost_target_start] // monster lost target function

AS_NONE 	[0]
AS_MISSILE 	[2]
AS_STRAIGHT 	[3]
attack_delay 	[0]
lost_time 	[0]
back_up 	[false]
back_time 	[0]
left_time 	[0]
back_flag 	[false]
fire_delay 	[0]
skill_time 	[0]
attack_state 	[0]
headshot	[false]

// spawn pawn and do setup work

Spawn[ ()
{
	Console(true);

	BoxWidth(BOXWIDTH); // set bounding box width/depth
	AttributeOrder(HEALTHATTRIBUTE, HEALTH, "Death"); // give monster health
	HostilePlayer(HOSTILEPLAYER); // set who monster is hostile to
	HostileSame(HOSTILESAME);
	HostileDifferent(HOSTILEDIFFERENT);
	SetFOV(FOV); // set field of view
	SetGroup(GROUP); // assign a group to belong to
	FindTargetOrder(SIGHTDIST, "FoundTarget", DAMAGEATTRIBUTE); // seen a target to chase
	AddPainOrder("IdleCheckHeadshot1", 100); // show pain and check for a headshot
	AddTriggerOrder("IdleToAlert", ALERTTRIGGER, 0); // go to alert when triggered
	SetWeapon(PAWNWEAPON);
	NewOrder("Idle");
} ]

// idle in place waiting for something to happen

Idle[ ()
{
	PlayAnimation(STAND, true, "");
	if(random(1,10)>6)
	{
		if(random(1,10)>5)
		{
			Rotate(TURNL, 102, 0, 90, 0, "");
			PlayAnimation(STAND, true, "");
			Rotate(TURNR, 108, 0, -90, 0, "");
		}
		else
		{
			Rotate(TURNR, 108, 0, -90, 0, "");
			PlayAnimation(STAND, true, "");
			Rotate(TURNL, 102, 0, 90, 0, "");
		}
	}
	RestartOrder();
} ]

IdleCheckHeadshot1[ ()
{
	SetEventState(ALERTTRIGGER, true); // set trigger to go to alert
	LowLevel("IdleCheckHeadshot2");
} ]

IdleCheckHeadshot2[ ()
{
	check_headshot(); //check if headshot has occurred

	if(random(1,100)<PAINPERCENT)
	{
		HighLevel("IdlePain"); // in pain
	}

	return 0;
} ]

// show pain at idle then trigger to alert

IdlePain[ ()
{
	switch(random(1,4)) // chose between 4 animations
	{
		case 1
		{
			PlayAnimation(PAIN, true, "");
		}
		case 2
		{
			PlayAnimation(PAIN1, true, "");
		}
		case 3
		{
			PlayAnimation(PAIN2, true, "");
		}
		case 4
		{
			PlayAnimation(PAIN3, true, "");
		}
	}
	Return();
} ]

// start shifting from idle to alert

IdleToAlert[ ()
{
	FindTargetOrder(ALERTSIGHTDIST, "FoundTarget", DAMAGEATTRIBUTE); // increase viewing distance
	AddPainOrder("AlertCheckHeadshot1", 100); // show pain check for head shot
	AddTimerOrder(1, 10, "AlertToIdle"); // go to idle after 10 secs
	SetEventState(ALERTTRIGGER, false); // turn off alert trigger
	NewOrder("Alert");
} ]

// look around at alert looking for enemy

Alert[ ()
{
	PlayAnimation(STAND, true, "");
	if(random(1,10)>3) // turn around once in awhile
	{
		if(random(1,10)>5)
		{
			Rotate(TURNL, 102, 0, 90, 0, "");
			Rotate(TURNL, 102, 0, 90, 0, "");
			Rotate(TURNL, 102, 0, 90, 0, "");
			Rotate(TURNL, 102, 0, 90, 0, "");
		}
		else
		{
			Rotate(TURNR, 108, 0, -90, 0, "");
			Rotate(TURNR, 108, 0, -90, 0, "");
			Rotate(TURNR, 108, 0, -90, 0, "");
			Rotate(TURNR, 108, 0, -90, 0, "");
		}
	}
	RestartOrder();
} ]

// show pain at alert

AlertCheckHeadshot1[ ()
{
	LowLevel("AlertCheckHeadshot2");
} ]

AlertCheckHeadshot2[ ()
{
	check_headshot(); //check if headshot has occurred

	if(random(1,100)<PAINPERCENT)
	{
		HighLevel("AlertPain"); // in pain
	}

	return 0;
} ]

AlertPain[ ()
{
	switch(random(1,4)) // play on of 4 animations
	{
		case 1
		{
			PlayAnimation(PAIN, true, "");
		}
		case 2
		{
			PlayAnimation(PAIN1, true, "");
		}
		case 3
		{
			PlayAnimation(PAIN2, true, "");
		}
		case 4
		{
			PlayAnimation(PAIN3, true, "");
		}
	}
	Return();
} ]

// timed out at alert so go to idle

AlertToIdle[ ()
{
	FindTargetOrder(SIGHTDIST, "FoundTarget", DAMAGEATTRIBUTE); // decrease viewing distance
	AddPainOrder("IdleCheckHeadshot1", 100); // show pain
	AddTriggerOrder("IdleToAlert", ALERTTRIGGER, 0); // go to alert when triggered
	NewOrder("Idle");
} ]

// found a target to attack

FoundTarget[ ()
{
	DelTimerOrder(1); // get rid of alert timer
	LowLevel(RUNFUNC); // attack functions are low level
} ]

// lost target while attacking so go back to idle again

LostTarget[ ()
{
	FindTargetOrder(SIGHTDIST, "FoundTarget", DAMAGEATTRIBUTE); // seen a target to chase
	AddPainOrder("IdleCheckHeadshot1", 100); // show pain and trigger alert
	AddTriggerOrder("IdleToAlert", ALERTTRIGGER, 0); // go to alert when triggered
	NewOrder("Idle");
} ]


check_headshot[ ()
{
	self.ThinkTime = 0.1;

	if(GetLastBoneHit()=HEADBONE)
	{
		SetAttribute(HEALTHATTRIBUTE,0); //kill pawn when head is hit
		headshot = true;
	}
	else
	{
		headshot = false;
	}

	return 0;
} ]


// you died

Death[ ()
{
	DelTimerOrder(1); // remove alert timer
	AddPainOrder("IdlePain", 0); // remove pain order
	FindTargetOrder(0, "FoundTarget", DAMAGEATTRIBUTE); // remove target finding
	DelTriggerOrder("IdleToAlert"); // remove alert trigger
	SetNoCollision(); // remove bounding box so there are no collisions with corpse

	if(headshot = true)
	{
		AnimateStop(DIEHEADSHOT, 0, "");
	}
	else
	{
		switch(random(1,4)) // chose between 4 death animations
		{
			case 1
			{
				AnimateStop(DIE, 0, "");
			}
			case 2
			{
				AnimateStop(DIE1, 0, "");
			}
			case 3
			{
				AnimateStop(DIE2, 0, "");
			}
			case 4
			{
				AnimateStop(DIE, 0, "");
			}
		}
	}

	LowLevel("Loot");
} ]

Loot[ ()
{
	self.thinktime = 0.1;

	if(self.player_range < 50)
	{
		ModifyAttribute(LOOTWEAPON,1);
		ModifyAttribute(LOOTAMMO,random(LOOTAMMOMIN,LOOTAMMOMAX));
		PlaySound(LOOTSOUND);

		HighLevel("Fade");
	}
} ]

Fade[ ()
{
	RemoveWeapon();
	Delay("",DIEHOLD,"");
	FadeOut(DIEFADE, 0); // fade out corpse
	Remove(true); // remove actor
} ]

// Low level attack routines

// Start of run to attack

monster_run_start[ ()
{
	Animate(RUN); // play run animation
	self.ThinkTime = 0; // start thinking on next frame
	self.think = "monster_run"; // go to run attack routine
	self.ideal_yaw = enemy_yaw; // set direction to run
	self.yaw_speed = YAWSPEED; // set rotation speed
	back_up = false; // initialize obsticle avoidance
	attack_state = AS_NONE; // not attacking yet
} ]

// run to enemy to attack

monster_run[ ()
{
	self.ThinkTime = 0.1;
	if(self.health<=0)
	{
		HighLevel("Death"); // dead
		return 0;
	}
	if(self.in_pain = true)
	{
		check_headshot();

		if(random(1,100)<PAINPERCENT)
		{
			self.think = "monster_run_pain_start"; // in pain
			return 0;
		}
	}
	if(EnemyExist(DAMAGEATTRIBUTE) < 3)
	{
		HighLevel("LostTarget"); // enemy is gone or dead
		return 0;
	}
	if(enemy_vis = false)
	{
		self.think = LOSTFUNC; // lost sight of enemy
		lost_time = time + LOSTTIME;
		return 0;
	}
	ai_run(random(RUNSPEED-2,RUNSPEED+2)); // run toward enemy
} ]

// start of pain while running

monster_run_pain_start[ ()
{
	Animate(PAIN); // play pain animation
	SetHoldAtEnd(true); // set to stop at animation end
	self.ThinkTime = 0.1;
	self.think = "monster_run_pain";
} ]

// wait for animation to stop

monster_run_pain[ ()
{
	self.ThinkTime = 0.1;
	if(self.animate_at_end = true) // wait for end of animation
	{
		self.think = "monster_run_start"; // start running again
		SetHoldAtEnd(false); // remove animation stop
		self.ThinkTime = 0;
	}
} ]

// start of lost sight of enemy routine

monster_lost_target_start[ ()
{
	Animate(RUN); // play run animation
	self.ThinkTime = 0; // start thinking on next frame
	self.think = "monster_lost_target";
} ]

// go to last known location of enemy

monster_lost_target[ ()
{
	self.ThinkTime = 0.1;
	if(lost_time<time)
	{
		HighLevel("LostTarget"); // timed out while looking
		return 0;
	}
	if(self.health<=0)
	{
		HighLevel("Death"); // died
		return 0;
	}
	if(self.in_pain = true)
	{
		check_headshot();

		if(random(1,100)<PAINPERCENT)
		{
			self.think = "monster_lost_pain_start"; // in pain
			return 0;
		}
	}
	if(EnemyExist(DAMAGEATTRIBUTE) < 3)
	{
		HighLevel("LostTarget"); // enemy died or was removed
		return 0;
	}
	if(enemy_vis = true)
	{
		self.think = "monster_run_start"; // seen again so run to attack
		self.ThinkTime = 0;
		return 0;
	}
	if(enemy_range>POINTRADIUS) // get close to last known location
	{
		walk_movetogoal(random(RUNSPEED-2,RUNSPEED+2));
	}
	else
	{
		HighLevel("LostTarget"); // can't find at last known location
		return 0;
	}
} ]

// start of showing pain while searching

monster_lost_pain_start[ ()
{
	Animate(PAIN); // play pain animation
	SetHoldAtEnd(true); // set to stop at end
	self.ThinkTime = 0.1;
	self.think = "monster_lost_pain";
} ]

// wait till animation is done

monster_lost_pain[ ()
{
	self.ThinkTime = 0.1;
	if(self.animate_at_end = true) // animation done
	{
		self.think = "monster_lost_target_start"; // go back to finding target
		SetHoldAtEnd(false); // remove stop at end
		self.ThinkTime = 0;
	}
} ]

// start of missile attack

monster_missile_start[ ()
{
	Animate(MISSILEATTACK); // play missile attack animation
	SetHoldAtEnd(true);
	self.ThinkTime = 0;
	self.think = "monster_missile";
	fire_delay = time + FIREDELAY; // set firing delay
	UpdateTarget(); // update target location
	skill_time = time + (SKILL*0.1); // calculate next update time
	attack_delay = time + ATTACKDELAY; // delay until next shot
} ]

// attack target with projectile

monster_missile[ ()
{
	self.ThinkTime = 0.1;
	if(self.health<=0)
	{
		SetHoldAtEnd(false);
		HighLevel("Death"); // dead
		return 0;
	}
	if(self.in_pain = true)
	{
		check_headshot();

		if(random(1,100)<PAINPERCENT)
		{
			self.think = "monster_missile_pain_start"; // in pain
			return 0;
		}
	}
	exist = EnemyExist(DAMAGEATTRIBUTE);
	if(exist < 2)
	{
		SetHoldAtEnd(false);
		HighLevel("LostTarget"); // enemy is dead and gone
		return 0;
	}
	if(exist = 2)
	{
		SetHoldAtEnd(false);
		HighLevel("LostTarget"); // enemy is dead but body remains
		return 0;
	}
	if(enemy_vis = false)
	{
		self.think = LOSTFUNC; // lost sight of enemy
		lost_time = time + LOSTTIME;
		SetHoldAtEnd(false);
		return 0;
	}
	if(enemy_range>MISSILERANGE)
	{
		self.think = RUNFUNC; // too far away so run toward
		SetHoldAtEnd(false);
		return 0;
	}
	if(skill_time<time) // update according to skill level
	{
		UpdateTarget(); // update target location
		skill_time = time + (SKILL*0.1);
		ai_face(); // face enemy while attacking
	}

	if(fire_delay<time) // delay after animation starts before firing
	{
		FireProjectile(PROJECTILE, FIREBONE, OFFSETX, OFFSETY, OFFSETZ, DAMAGEATTRIBUTE);
		fire_delay = time + 1000; // set delay well ahead so it is ignored
		PlaySound(ATTACKSOUND);
	}

	// wait until animation is done before attacking again
	// hold animation at end until attack delay is over

	if(self.animate_at_end = true)
	{
		if(attack_delay < time)
		{
			self.think = "monster_missile_start"; // go back to missile attack
			SetHoldAtEnd(false);
			self.ThinkTime = 0.1;
		}
	}
} ]

// start of showing pain

monster_missile_pain_start[ ()
{
	Animate(PAIN); // play pain animation
	SetHoldAtEnd(true); // set to stop at end
	self.ThinkTime = 0.1;
	self.think = "monster_missile_pain";
} ]

// wait until animation is done

monster_missile_pain[ ()
{
	self.ThinkTime = 0.1;
	if(self.animate_at_end = true) // animation is done
	{
		self.think = "monster_missile_start"; // go back to missile attack
		SetHoldAtEnd(false);
		self.ThinkTime = 0;
	}
} ]




// basic AI routines

// run toward enemy and see if you are ready to attack

ai_run[ (dist)
{
	if (attack_state = AS_MISSILE) // do missile attack
	{
		ai_run_missile();
		return 0;
	}
	if (CheckAnyAttack()) // check if you can start the actual attack
	{
		return 0;
	}
	walk_movetogoal(dist); // else move toward the enemy
} ]

// missile attack setup

ai_run_missile[ ()
{
	ai_face();
	if(FacingIdeal()) // got close enough
	{
		self.think = MISSILEFUNC; // start missile attack
		attack_state = AS_STRAIGHT;
		UpdateTarget();
		skill_time = time + (SKILL*0.1);
	}
} ]

// face enemy

ai_face[ ()
{
	self.ideal_yaw = enemy_yaw;
	ChangeYaw(); // rotate to face enemy
} ]

// use walkmove to naviagte to enemy

walk_movetogoal[ (dist)
{
	if(IsFalling = true)
	{
		return 0; // don't move while falling
	}
	if(back_up = false)
	{
		ai_face(); // turn to face enemy
		if(FacingIdeal())
		{
			if(walkmove(self.current_yaw, dist) = true)
			{
				return 0; // can move in current direction
			}
			else
			{
				if(random(1,10)<3) // backup and move sideways
				{
					back_up = true;
					back_time = time + 0.5;
					back_flag = false;
					return 0;
				}
				else
				{
					ForceUp(30); // jump up, forward and to side
					ForceForward(30);
					if(random(1,10)<6)
					{
						ForceRight(30);
					}
					else
					{
						ForceLeft(30);
					}
				}
			}
		}
	}
	else
	{
		if(back_flag = false) // go backward 1/2 sec
		{
			if(back_time > time)
			{
				walkmove((self.current_yaw-(180*0.0174532925199433)), dist);
				return 0;
			}
			else
			{
				back_time = time + 0.5;
				back_flag = true;
			}
		}
		if(back_time > time) // go sideways 1/2 sec
		{
			walkmove((self.current_yaw-(90*0.0174532925199433)), dist);
			return 0;
		}
		back_up = false;
	}
} ]


// check if nearly facing enemy

FacingIdeal[ ()
{
	selfangle = self.current_yaw/0.0174532925199433; // your direction in degrees
	idealangle = self.ideal_yaw/0.0174532925199433; // his direction in degrees
	delta = selfangle - idealangle; // difference in directions
	if (delta > -20 and delta < 20) // within 20 degrees is close enough
	{
		return true;
	}
	return false;
} ]

// check if ready to do actual attacking

CheckAnyAttack[ ()
{
	if(attack_delay>time)
	{
		return false;
	}
	if(enemy_range<MISSILERANGE) // inside missile range
	{
		attack_state = AS_MISSILE; // do a missile attack
		return true;
	}
	return false;
} ]

}